﻿using System;
using System.Net;
using System.Text;
using Newtonsoft.Json;
using RestSharp;

namespace appointmentReminder.Class
{
	public class MessageHandler
	{
		private String baseUrl = "https://livelink.sapmobileservices.com/api";
		private String appKey;
		private String appSecret;
		private String defaultOrigin;
		private WebProxy globalProxy;

		public MessageHandler()
		{
		}

		public MessageHandler(String appKey, String appSecret, String defaultOrigin = "", WebProxy proxy = null)
		{
			this.appKey = appKey;
			this.appSecret = appSecret;
			this.defaultOrigin = defaultOrigin;
			this.globalProxy = proxy;
		}

		public String GetAccessToken()
		{
			String oAuthUrl = this.baseUrl + "/oauth/token";
			String bearer = Convert.ToBase64String(Encoding.UTF8.GetBytes(this.appKey + ":" + this.appSecret));
			RestClient client = new RestClient(oAuthUrl);
			if (this.globalProxy != null) {
				client.Proxy = this.globalProxy;
				client.Proxy.Credentials = System.Net.CredentialCache.DefaultCredentials;
			}
			RestRequest request = new RestRequest(Method.POST);
			request.AddHeader("content-type", "application/x-www-form-urlencoded");
			request.AddHeader("authorization", "Basic " + bearer);
			request.AddParameter("application/x-www-form-urlencoded", "grant_type=client_credentials", ParameterType.RequestBody);

			IRestResponse<oAuthToken> response = client.Execute<oAuthToken>(request);

			if (response.StatusCode != HttpStatusCode.OK) {
				Console.WriteLine("..... Error: " + response.StatusDescription);
				return "";
			}

			String accessToken = response.Data.access_token;
			return accessToken;
		}

		public void sendMessage(String message, String destination, String origin = "")
		{
			String messageUrl = this.baseUrl + "/v2/sms";
			Object messagePayload = new {
				message = message,
				origin = origin == "" ? this.defaultOrigin : origin,
				destination = destination
			};
			String messageData = JsonConvert.SerializeObject(messagePayload);
			String bearer = GetAccessToken();
			if (String.IsNullOrEmpty(bearer)) {
				return;
			}
			RestClient client = new RestClient(messageUrl);
			if (this.globalProxy != null) {
				client.Proxy = this.globalProxy;
				client.Proxy.Credentials = System.Net.CredentialCache.DefaultCredentials;
			}

			RestRequest request = new RestRequest(Method.POST);
			request.AddHeader("authorization", "Bearer " + bearer);
			request.AddHeader("content-type", "application/json");
			request.AddParameter("application/json", messageData, ParameterType.RequestBody);

			IRestResponse response = client.Execute(request);

			if (response.StatusCode != HttpStatusCode.OK) {
				Console.WriteLine("..... failed to send message.");
				Console.WriteLine("..... Error: " + response.ErrorMessage);
			}
		}
	}
}